# Set simulation parameters
source("aux.R")
nsim <- 10000								# Number of simulation iterations per condition.
alpha <- 0.05								# Probability of a Type I error (assuming two-sided).
test.stat.threshold <- qnorm(1-alpha/2)		# z-value correspondeing to one-sided alpha
sigma <- 1									# Individual-level variance (wlog set to one so effect
	 										# size is on Cohen's d scale).

d <- seq(0.1,0.9,by=0.1)					# Effect size on Cohen's d scale.
p.report <- c(1,0) 							# Probability of reporting a study which is not both
	 										# directionally consistent and stat. significant.
n.study <- seq(10, 100, by=10)				# Number of studies available.
power <- seq(0.1,0.9,by=0.1)				# Power per study.

# Intervals to search over for grid-based optimization methods
gsi	 <- c(-0.5, 2)							
gsi.low <- c(-5, 0)
gsi.high <- c(1.5, 5)

# Run simulation
sim.param <- expand.grid(d,p.report,n.study,power)
names(sim.param) <- c("d", "p.report", "n.study", "power")
sim.param$n.per.cell <- NA
for(i in 1:nrow(sim.param)){
	sim.param$n.per.cell[i] <- ceiling(get.n(sim.param$d[i], sim.param$power[i], alpha/2))	
}
sim.data <- list(NULL)
sim.count <- matrix(NA, nrow(sim.param), nsim)
d.hat <- d.se <- d.ci.lb <- d.ci.ub <- 
	ap.hat <- ap.ci.lb <- ap.ci.ub <- matrix(NA, nrow(sim.param), nsim)
ap <- rep(NA, nrow(sim.param))
for(i in 1:nrow(sim.param)){
	sim.data[[i]] <- list(NULL)	
	ap[i] <- average.power( sim.param$d[i], sim.param$n.per.cell[i], sim.param$n.per.cell[i], alpha/2 )
	
	for(j in 1:nsim){
		sim.data[[i]][[j]] <- list(NULL)
		
		# Generate data
		test.stat <- NULL
		while(length(test.stat) < sim.param$n.study[i]){
			tmp.stat <- rnorm(1, sim.param$d[i]/sqrt(2/sim.param$n.per.cell[i]), 1)
			tmp.bern <- rbinom(1, 1, sim.param$p.report[i])
			if(tmp.bern==1 | tmp.stat>test.stat.threshold){ 
				test.stat <- c(test.stat, tmp.stat) }
		}
		nn <- rep(sim.param$n.per.cell[i], length(test.stat))
		
		# Fit method: MLE (extreme one sided selection; homogeneous)
		# or
		# Fit method: MLE (no selection; homogeneous)
		mm <- NULL
		if(sim.param$p.report[i]==0){
			try( mm <- estimate.z.extreme.selection.homogeneous(test.stat, nn, nn, alpha/2, gsi), silent=TRUE )
			# Check for numerical optimization problems; if so, attempt to resolve them
			if(!is.null(mm)){
				if(mm[1] < gsi[1] + 0.01){ 
					mm1 <- mm2 <- NULL
					try( mm1 <- estimate.z.extreme.selection.homogeneous(test.stat, nn, nn, alpha/2, gsi.low), silent=TRUE )
					try( mm2 <- estimate.z.extreme.selection.homogeneous(test.stat, nn, nn, alpha/2, gsi.low+c(-0.0001,0.0001)), silent=TRUE )
					if(!is.null(mm1) & is.null(mm2)){ mm <- mm1 }
					if(is.null(mm1) & !is.null(mm2)){ mm <- mm2 }
					if(!is.null(mm1) & !is.null(mm2)){
						if(mm1[3] >= mm2[3]){ mm <- mm1 }else{ mm <- mm2 }
					}
				}
				if(mm[1] > gsi[2] - 0.01){ 
					mm1 <- mm2 <- NULL
					try( mm1 <- estimate.z.extreme.selection.homogeneous(test.stat, nn, nn, alpha/2, gsi.high), silent=TRUE )
					try( mm2 <- estimate.z.extreme.selection.homogeneous(test.stat, nn, nn, alpha/2, gsi.high+c(-0.0001,0.0001)), silent=TRUE )
					if(!is.null(mm1) & is.null(mm2)){ mm <- mm1 }
					if(is.null(mm1) & !is.null(mm2)){ mm <- mm2 }
					if(!is.null(mm1) & !is.null(mm2)){
						if(mm1[3] >= mm2[3]){ mm <- mm1 }else{ mm <- mm2 }
					}
				}
			}
		}
		if(sim.param$p.report[i]==1){
			# These two are equivalent but the latter is analytic and faster
			#try( mm <- estimate.z.no.selection.homogeneous(test.stat, nn, nn, gsi), silent=TRUE )
			try( mm <- meta.fe(test.stat*sqrt(2/nn), 2/nn), silent=TRUE )
		}
		
		# Save results
		if(!is.null(mm)){
			d.hat[i,j] <- mm[1]
			d.se[i,j] <- mm[2]
			d.ci.lb[i,j] <- mm[1] + qnorm(alpha/2)*mm[2]
			d.ci.ub[i,j] <- mm[1] + qnorm(1-alpha/2)*mm[2]
			ap.hat[i,j] <- average.power( d.hat[i,j], nn, nn, alpha/2 )
			ap.ci.lb[i,j] <- average.power( d.ci.lb[i,j], nn, nn, alpha/2 )
			ap.ci.ub[i,j] <- average.power( d.ci.ub[i,j], nn, nn, alpha/2 )
		}
		
		# Save data in case of error
		if(is.null(mm) || is.na(mm[2])){ sim.data[[i]][[j]] <- test.stat }
	}
	if(i%%25==0){ cat(i,"... ") }
}
save.image("output.Rdata")
